import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import type { TypeParameterReflection } from "../../../../models";
import { JSX } from "../../../../utils";
export declare function typeParameters(context: DefaultThemeRenderContext, typeParameters: TypeParameterReflection[]): JSX.Element;
