"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommentTagSerializer = exports.CommentSerializer = void 0;
var comment_1 = require("./comment");
Object.defineProperty(exports, "CommentSerializer", { enumerable: true, get: function () { return comment_1.CommentSerializer; } });
var comment_tag_1 = require("./comment-tag");
Object.defineProperty(exports, "CommentTagSerializer", { enumerable: true, get: function () { return comment_tag_1.CommentTagSerializer; } });
