import { QueryType } from "../../../models";
import { TypeSerializerComponent } from "../../components";
import type { QueryType as JSONQueryType } from "../../schema";
export declare class QueryTypeSerializer extends TypeSerializerComponent<QueryType> {
    supports(t: unknown): boolean;
    toObject(type: QueryType, obj: Pick<JSONQueryType, "type">): JSONQueryType;
}
