import { Reflection } from "../../../models";
import { ReflectionSerializerComponent } from "../../components";
import type { Reflection as JSONReflection } from "../../schema";
export declare class ReflectionSerializer extends ReflectionSerializerComponent<Reflection> {
    static PRIORITY: number;
    supports(t: unknown): boolean;
    toObject(reflection: Reflection, obj?: Partial<JSONReflection>): JSONReflection;
}
