"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommentTag = exports.Comment = void 0;
var comment_1 = require("./comment");
Object.defineProperty(exports, "Comment", { enumerable: true, get: function () { return comment_1.Comment; } });
var tag_1 = require("./tag");
Object.defineProperty(exports, "CommentTag", { enumerable: true, get: function () { return tag_1.CommentTag; } });
