import { ReferenceReflection } from "../../../models";
import { ReflectionSerializerComponent } from "../../components";
import type { DeclarationReflection as JSONDeclarationReflection, ReferenceReflection as JSONReferenceReflection } from "../../schema";
export declare class ReferenceReflectionSerializer extends ReflectionSerializerComponent<ReferenceReflection> {
    static PRIORITY: number;
    supports(t: unknown): boolean;
    toObject(ref: ReferenceReflection, obj: JSONDeclarationReflection): JSONReferenceReflection;
}
