"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConverterComponent = exports.Component = void 0;
const component_1 = require("../utils/component");
Object.defineProperty(exports, "Component", { enumerable: true, get: function () { return component_1.Component; } });
class ConverterComponent extends component_1.AbstractComponent {
}
exports.ConverterComponent = ConverterComponent;
