import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import type { ContainerReflection } from "../../../../models";
export declare function members(context: DefaultThemeRenderContext, props: ContainerReflection): JSX.Element;
