import type { SourceReference } from "../../../models";
/**
 * An internal concrete implementation for the [[ SourceReference ]] interface
 * so it can be identified
 */
export declare class SourceReferenceWrapper {
    sourceReference: SourceReference;
    constructor(sourceReference: SourceReference);
}
