import { DeclarationReflection } from "../../../../models";
import { JSX } from "../../../../utils";
import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
export declare const memberDeclaration: (context: DefaultThemeRenderContext, props: DeclarationReflection) => JSX.Element;
