import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
export declare function analytics(context: DefaultThemeRenderContext): JSX.Element | undefined;
