"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UrlMapping = void 0;
class UrlMapping {
    constructor(url, model, template) {
        this.url = url;
        this.model = model;
        this.template = template;
    }
}
exports.UrlMapping = UrlMapping;
