import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { Type } from "../../../../models";
import { JSX } from "../../../../utils";
export declare function type(context: DefaultThemeRenderContext, type: Type | undefined): JSX.Element;
