import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import type { DeclarationReflection, SignatureReflection } from "../../../../models";
export declare const memberSources: (context: DefaultThemeRenderContext, props: SignatureReflection | DeclarationReflection) => JSX.Element;
