"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecoratorWrapper = exports.SourceReferenceWrapper = void 0;
var source_reference_wrapper_1 = require("./source-reference-wrapper");
Object.defineProperty(exports, "SourceReferenceWrapper", { enumerable: true, get: function () { return source_reference_wrapper_1.SourceReferenceWrapper; } });
var decorator_wrapper_1 = require("./decorator-wrapper");
Object.defineProperty(exports, "DecoratorWrapper", { enumerable: true, get: function () { return decorator_wrapper_1.DecoratorWrapper; } });
