import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import type { PageEvent } from "../../../events";
import { ContainerReflection } from "../../../../models";
import { JSX } from "../../../../utils";
export declare const reflectionTemplate: (context: DefaultThemeRenderContext, props: PageEvent<ContainerReflection>) => JSX.Element;
