import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import { SignatureReflection } from "../../../../models";
export declare const memberSignatureBody: (context: DefaultThemeRenderContext, props: SignatureReflection, { hideSources }?: {
    hideSources?: boolean | undefined;
}) => JSX.Element;
