import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import type { ReferenceReflection } from "../../../../models";
export declare const memberReference: ({ urlTo }: DefaultThemeRenderContext, props: ReferenceReflection) => JSX.Element;
