"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RepositoryType = void 0;
var RepositoryType;
(function (RepositoryType) {
    RepositoryType["GitHub"] = "github";
    RepositoryType["Bitbucket"] = "bitbucket";
    RepositoryType["GitLab"] = "gitlab";
})(RepositoryType = exports.RepositoryType || (exports.RepositoryType = {}));
