"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceReferenceContainerSerializer = void 0;
var source_reference_1 = require("./source-reference");
Object.defineProperty(exports, "SourceReferenceContainerSerializer", { enumerable: true, get: function () { return source_reference_1.SourceReferenceContainerSerializer; } });
