import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import type { ReflectionGroup } from "../../../../models";
export declare function membersGroup(context: DefaultThemeRenderContext, group: ReflectionGroup): JSX.Element;
