import type { Reflection } from "../../../../models";
import { JSX } from "../../../../utils";
import type { PageEvent } from "../../../events";
import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
export declare function footer(context: DefaultThemeRenderContext, props: PageEvent<Reflection>): JSX.Element;
