"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.memberGetterSetter = void 0;
const utils_1 = require("../../../../utils");
const memberGetterSetter = (context, props) => (utils_1.JSX.createElement(utils_1.JSX.Fragment, null,
    utils_1.JSX.createElement("ul", { class: "tsd-signatures " + props.cssClasses },
        !!props.getSignature && (utils_1.JSX.createElement("li", { class: "tsd-signature tsd-kind-icon" },
            utils_1.JSX.createElement("span", { class: "tsd-signature-symbol" }, "get"),
            " ",
            props.name,
            context.memberSignatureTitle(props.getSignature, { hideName: true }))),
        !!props.setSignature && (utils_1.JSX.createElement("li", { class: "tsd-signature tsd-kind-icon" },
            utils_1.JSX.createElement("span", { class: "tsd-signature-symbol" }, "set"),
            " ",
            props.name,
            context.memberSignatureTitle(props.setSignature, { hideName: true })))),
    utils_1.JSX.createElement("ul", { class: "tsd-descriptions" },
        !!props.getSignature && utils_1.JSX.createElement("li", { class: "tsd-description" }, context.memberSignatureBody(props.getSignature)),
        !!props.setSignature && utils_1.JSX.createElement("li", { class: "tsd-description" }, context.memberSignatureBody(props.setSignature)))));
exports.memberGetterSetter = memberGetterSetter;
