import { ReflectionType } from "../../../models";
import { TypeSerializerComponent } from "../../components";
import type { ReflectionType as JSONReflectionType } from "../../schema";
export declare class ReflectionTypeSerializer extends TypeSerializerComponent<ReflectionType> {
    supports(t: unknown): boolean;
    toObject(reference: ReflectionType, obj: Pick<JSONReflectionType, "type">): JSONReflectionType;
}
