"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTypeDocOptions = void 0;
var typedoc_1 = require("./typedoc");
Object.defineProperty(exports, "addTypeDocOptions", { enumerable: true, get: function () { return typedoc_1.addTypeDocOptions; } });
