import { ReferenceType } from "../../../models";
import { TypeSerializerComponent } from "../../components";
import type { ReferenceType as JSONReferenceType } from "../../schema";
export declare class ReferenceTypeSerializer extends TypeSerializerComponent<ReferenceType> {
    supports(t: unknown): boolean;
    toObject(type: ReferenceType, obj: Pick<JSONReferenceType, "type"> & Partial<JSONReferenceType>): JSONReferenceType;
}
