export declare const ConverterEvents: {
    readonly BEGIN: "begin";
    readonly END: "end";
    readonly CREATE_DECLARATION: "createDeclaration";
    readonly CREATE_SIGNATURE: "createSignature";
    readonly CREATE_PARAMETER: "createParameter";
    readonly CREATE_TYPE_PARAMETER: "createTypeParameter";
    readonly RESOLVE_BEGIN: "resolveBegin";
    readonly RESOLVE: "resolveReflection";
    readonly RESOLVE_END: "resolveEnd";
};
