import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import type { DeclarationHierarchy } from "../../../../models";
export declare const hierarchy: (context: DefaultThemeRenderContext, props: DeclarationHierarchy) => JSX.Element;
