import { ParameterReflection } from "../../../models";
import { ReflectionSerializerComponent } from "../../components";
import type { ParameterReflection as JSONParameterReflection, Reflection as JSONReflection } from "../../schema";
export declare class ParameterReflectionSerializer extends ReflectionSerializerComponent<ParameterReflection> {
    supports(t: unknown): boolean;
    toObject(parameter: ParameterReflection, obj: JSONReflection): JSONParameterReflection;
}
