import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { Type } from "../../../../models";
import { JSX } from "../../../../utils";
export declare const typeAndParent: (context: DefaultThemeRenderContext, props: Type) => JSX.Element;
