import type { DefaultThemeRenderContext } from "../DefaultThemeRenderContext";
import { JSX } from "../../../../utils";
import type { Reflection } from "../../../../models";
export declare const breadcrumb: (context: DefaultThemeRenderContext, props: Reflection) => JSX.Element | undefined;
