import { DeclarationReflection } from "../../../models";
import { ReflectionSerializerComponent } from "../../components";
import type { DeclarationReflection as JSONDeclarationReflection, ContainerReflection as JSONContainerReflection } from "../../schema";
export declare class DeclarationReflectionSerializer extends ReflectionSerializerComponent<DeclarationReflection> {
    static PRIORITY: number;
    supports(t: unknown): boolean;
    toObject(d: DeclarationReflection, obj: JSONContainerReflection): JSONDeclarationReflection;
}
